//test app #2; 2009 qrleon

#include <stdlib.h>
#include <time.h>
#include <curses.h>

//  some extra keycodes. might not be portable?
#define KEYB_TAB 9
#define KEYB_CTRL_LEFT 443
#define KEYB_CTRL_RIGHT 444
#define KEYB_CTRL_UP 480
#define KEYB_CTRL_DOWN 481

// game states
#define PLAYING 0
#define EDITING 1

// map things
#define A_BLANK        0
#define A_WALL         1
#define A_MINE         2
#define A_SANDWICH     3
#define A_PLAYER_START 4

// movement input
#define MOVE_LEFT          1
#define MOVE_RIGHT         2
#define MOVE_JUMP_LEFT     3
#define MOVE_JUMP_RIGHT    4

#define MOVE_HI_JUMP_LEFT  5
#define MOVE_HI_JUMP_RIGHT 6
#define MOVE_HI_JUMP       7

#define MOVE_MEASURE       8
#define MOVE_STOP          9

// *** reminder: in PDCurses functions, Y goes before X ***

#define MAX_STORED_MAPS 9
#define SCREEN_Y 24
#define SCREEN_X 80

// for tracing jumps
#define P_PREV_MAX 160

// transitions
#define TRIPPED_A_MINE 1
#define FELL_TO_DEATH 2
#define WON_LEVEL 4
#define WON_GAME 5

int isItOverYet = 0;

chtype g_player, g_shadowPlayer, g_shadowPlayerHurt;
chtype g_wall, g_empty, g_mine, g_sandwich;



int mapStore[ MAX_STORED_MAPS ][ SCREEN_Y ][ SCREEN_X ];
int map[ SCREEN_Y ][ SCREEN_X ];
int mapIndex = 0;

int quit = 0;
int appMode = PLAYING;


int keyIn = 0;

int pY = 0, pX = 0;
int pStartY = 0, pStartX = 0;

//movement sequence
int pMoveRise = 0, pMoveSide = 0;
int pMoveAugment = 0;

int pHealth = 0, pTurns = 0;
int startHealth = 20, healthMax = 40;

int pInput = 0, pFinishInput = 0;

int pFallDamage;
int wasHurt = 0; //to emphasize that the player was hurt from a long fall...

int eY = 0, eX = 0;
int eYMove = 0, eXMove = 0;

int eThingIndex = 0;
int eThingLen = 4;
int ePaintMode = 0;
int eFastMoveDist = 5;

//numbers that will be used to draw motion
int pPrevY[ P_PREV_MAX ];
int pPrevX[ P_PREV_MAX ];
int pPrevLen = 0;
int pPrevHurt = P_PREV_MAX + 1;

int wannaTracePossibleJumps = 0;

int wannaSave = 0;
int wannaLoad = 0;

int wannaPlot = 0;
int wannaFill = 0;
int wannaGrab = 0;

////////////////////////////////////////

int init();

int load_screen_from_text_file();
int save_screen_to_text_file();

void title_screen();

void reset_level();
void transition( int cause );

void record_previous_coordinates();
void blank_previous_coordinates();

void change_app_mode( int );
void win();

void check_plot_thing();
void flood_fill();

void draw_screen();
void trace_possible_jump( int y, int x );

void logic();

int map_collision( int y, int x );

void draw();
void input();

void logic_edit();
void draw_edit();
void input_edit();

void play();
void edit();

void clean_up();

////////////////////////////////////////

int init()
{
    //begin PDCurses setup -------------------------------------------

    srand( time( NULL ) );

    initscr();       //first PDCurses call
    curs_set( 0 );   //hide the cursor
    start_color();   //set up PDCurses color routines...

    int i = 0, f = 0, b = 0;

    //this should initialize 64 color-pair types, right?
    //    the PDCurses instructions warn not to use numbers to initialize.
    //    but just for this one program, it should be ok...
    for( b = 0; b < 8; b++ )
    {
        for( f = 0; f < 8; f++ )
        {
            init_pair( i, f, b );
            i++;
        }
    }

    keypad( stdscr, 1 );
    raw();
    nodelay( stdscr, 0 );
    noecho();
    nonl();

    //end PDCurses setup -------------------------------------------

    //wipe map arrays
    int z, y, x;
    for( z = 0; z < MAX_STORED_MAPS; z++ )
    {
        for( y = 0; y < SCREEN_Y; y++ )
        {
            for( x = 0; x < SCREEN_X; x++ )
            {
                mapStore[ z ][ y ][ x ] = 0;
            }
        }
    }
    for( y = 0; y < SCREEN_Y; y++ )
    {
        for( x = 0; x < SCREEN_X; x++ )
        {
            map[ y ][ x ]= 0;
        }
    }

    blank_previous_coordinates();   //zero out pPrev numbers

              g_player = '@' | COLOR_PAIR( 14 ) | A_BOLD;
        g_shadowPlayer = '@' | COLOR_PAIR( 4 );
    g_shadowPlayerHurt = '@' | COLOR_PAIR( 4 ) | A_BOLD;
                g_wall = '&' | COLOR_PAIR( 3 );
               g_empty = '.' | COLOR_PAIR( 7 );
                g_mine = 'X' | COLOR_PAIR( 4 ) | A_BOLD;
            g_sandwich = '=' | COLOR_PAIR( 6 );



    //center editor cursor
    eY = SCREEN_Y / 2;
    eX = SCREEN_X / 2;

    return 0;
}

void title_screen()
{
    erase();
                   //0         1         2         3         4         5         6         7
                   //01234567890123456789012345678901234567890123456789012345678901234567890123456789
    color_set( 4, NULL );
    mvprintw( 0, 0, "Glorious Trainwrecks Dot Com Presents:" );
    color_set( 7, NULL );
    mvprintw( 5, 15, "SSSSS   A   N   N DDDD  W   W IIIII  CCCC H   H" );
    mvprintw( 6, 14, "SS      A A  NN  N D   D W   W   I   C     H   H" );
    mvprintw( 7, 15, "SSSS  AAAAA N N N D   D W W W   I   C     HHHHH" );
    mvprintw( 8, 18, "SS A   A N  NN D   D W W W   I   C     H   H" );
    mvprintw( 9, 14, "SSSSS  A   A N   N DDDD   W W  IIIII  CCCC H   H" );
   mvprintw( 10, 13, "--------------------------------------------------" );
   mvprintw( 11, 15, "Hopper" );

   mvprintw( 14, 20, "\"Claim your sandwiches, my lord.\"" );
   mvprintw( 15, 20, "Use the NUMPAD to perform daring hops." );
   mvprintw( 16, 20, "Try not to hurt yourself MY LORD." );

   mvprintw( 18, 32, "1 and 3: Walk" );
   mvprintw( 19, 24, "4 and 6: Sprint jumps" );
   mvprintw( 20, 23, "7, 8 and 9: High jumps" );
   mvprintw( 22, 23, "0 may help you with jump distances." );
                   //--------------------------------------------------------------------------------
}

void reset_level()
{
    int y, x;
    //first, erase map
    for( y = 0; y < SCREEN_Y; y++ )
    {
        for( x = 0; x < SCREEN_X; x++ )
        {
            map[ y ][ x ] = 0;
        }
    }

    //copy map back from MapStore
    for( y = 0; y < SCREEN_Y; y++ )
    {
        for( x = 0; x < SCREEN_X; x++ )
        {
            map[ y ][ x ] = mapStore[ mapIndex ][ y ][ x ];
        }
    }

    //set up the player! fuck yes.

    //hunt down the player starting point
    pStartX = 0;
    pStartY = 0;
    for( y = 0; y < SCREEN_Y; y++ )
    {
        for( x = 0; x < SCREEN_X; x++ )
        {
            if( mapStore[ mapIndex ][ y ][ x ] == A_PLAYER_START )
            {
                pStartX = x;
                pStartY = y;
            }
        }
    }

    //blank player vars
    pY = pStartY;
    pX = pStartX;
    pMoveRise = 0;
    pMoveSide = 0;
    pHealth = startHealth;
    pTurns = 0;
    pMoveAugment = 0;
    pFallDamage =

    //reset the previous position tracer...
    pPrevLen = 0;
}

void transition( int cause )
{
    draw_screen();
    mvaddch( pY, pX, '!' | COLOR_PAIR( 4 ) | A_BOLD );
    switch( cause )
    {
        case TRIPPED_A_MINE:
        {
            mvprintw( 24, 0, "OUCH CRAP I STEPPED ON A MINE" );
            break;
        }
        case FELL_TO_DEATH:
        {
            mvprintw( 24, 0, "OH NO I HURT MY BUTT" );
            break;
        }
        case WON_LEVEL:
        {
            mvprintw( 24, 0, "I HAVE WON THE LEVEL" );
            break;
        }
        default:
        {
            break;
        }
    }
    if( cause != WON_GAME )
    {
        getch();
    }
    if( cause == WON_LEVEL )
    {
        if( mapIndex < MAX_STORED_MAPS )
        {
            mapIndex++;
        }
    }
    if( cause == WON_GAME )
        {
            win();
            quit = 1;
        }

    reset_level();
}

void blank_previous_coordinates()
{
    pPrevLen = 0;
    pPrevHurt = P_PREV_MAX + 1;

    int i;
    for( i = 0; i < P_PREV_MAX; i++ )
    {
        pPrevX[ i ] = 0;
        pPrevY[ i ] = 0;
    }
}



void check_plot_thing()
{
    if( wannaPlot == 1 )
    {
        mapStore[ mapIndex ][ eY ][ eX ] = eThingIndex;
        //copy mapStore info to on-display map
        reset_level();
    }
}

void flood_fill()
{
    if( mapStore[ mapIndex ][ eY ][ eX ] == eThingIndex )   //don't bother if the target thing and selected thing are the same.
    {
        return;
    }
    int targetThing = mapStore[ mapIndex ][ eY ][ eX ];     //thing to fill over


    int y, x;
    unsigned char mapFill[ SCREEN_Y ][ SCREEN_X ];

    for( y = 0; y < SCREEN_Y; y++ )     //initialize temp array
    {
        for( x = 0; x < SCREEN_X; x++ )
        {
            mapFill[ y ][ x ] = 0;
        }
    }

    mapFill[ eY ][ eX ] = 1;            //mark source for flooding

    //go over the array until there are no more 1's.
    int doneFlooding = 0;
    while( doneFlooding == 0 )
    {
        doneFlooding = 1;
        for( y = 0; y < SCREEN_Y; y++ )
        {
            for( x = 0; x < SCREEN_X; x++ )
            {
                if( mapFill[ y ][ x ] == 1 )
                {
                    doneFlooding = 0;
                    //if it's the correct thing, change it and mark the adjacent sides for checking.
                    if( mapStore[ mapIndex ][ y ][ x ] == targetThing )
                    {
                        mapStore[ mapIndex ][ y ][ x ] = eThingIndex;
                        if( y > 0 )
                        {
                            if( mapFill[ y - 1 ][ x ] == 0 )
                            {
                                mapFill[ y - 1 ][ x ] = 1;
                            }
                        }
                        if( y < SCREEN_Y - 1)
                        {
                            if( mapFill[ y + 1 ][ x ] == 0 )
                            {
                                mapFill[ y + 1 ][ x ] = 1;
                            }
                        }
                        if( x > 0 )
                        {
                            if( mapFill[ y ][ x - 1 ] == 0 )
                            {
                                mapFill[ y ][ x - 1 ] = 1;
                            }
                        }
                        if( x < SCREEN_X - 1)
                        {
                            if( mapFill[ y ][ x + 1 ] == 0 )
                            {
                                mapFill[ y ][ x + 1 ] = 1;
                            }
                        }
                    }
                    mapFill[ y ][ x ] = 2;
                }
            }
        }
    }
}


void change_app_mode( int newMode )
{
    reset_level();
    appMode = newMode;
}

void win()
{
    isItOverYet = 1;
    erase();
    int ii;
    for( ii = 0; ii < 24; ii++ )
    {
        mvprintw( ii, ii * 7, "YOU WIN" );
    }
    mvprintw( 0, 40, "Press a key to quit." );
    getch();
    return;
}

void draw_screen()
{
    erase();
    int x, y;
    for( y = 0; y < SCREEN_Y; y++ )
    {
        for( x = 0; x < SCREEN_X; x++ )
        {
            switch( map[ y ][ x ] )
            {
                case A_BLANK:
                {
                    mvaddch( y, x, g_empty );
                    break;
                }
                case A_PLAYER_START:
                {
                    mvaddch( y, x, '_' | COLOR_PAIR( 2 ) );
                    break;
                }
                case A_WALL:
                {
                    mvaddch( y, x, g_wall );
                    break;
                }
                case A_MINE:
                {
                    mvaddch( y, x, g_mine );
                    break;
                }
                case A_SANDWICH:
                {
                    mvaddch( y, x, g_sandwich );
                    break;
                }
            }
        }
    }

    //if there has been motion recently, mark it
    if( pPrevLen > 0 )
    {
        int i;
        for( i = 0; i < pPrevLen; i++ )
        {
            if( i < pPrevHurt )
            {
                mvaddch( pPrevY[ i ], pPrevX[ i ], g_shadowPlayer );
            }
            if( i >= pPrevHurt )
            {
                mvaddch( pPrevY[ i ], pPrevX[ i ], g_shadowPlayerHurt );
            }
        }
    }
    //if the player wants to see possible jumps, mark them.
    if( wannaTracePossibleJumps == 1 )
    {
        trace_possible_jump( pY - 1, pX );

        int i;
        for( i = 3 + pMoveAugment; i > 0; i-- )
        {
            // hops
            //   left
            trace_possible_jump( pY - 1, pX + i );
            //   right
            trace_possible_jump( pY - 1, pX - i );

            // hi-jumps
            //   jump height
            trace_possible_jump( pY - i, pX );
        }
        for( i = 0; i < 5; i++ )
        {
            trace_possible_jump( pY - (3 + pMoveAugment), (pX - 2) + i );
        }
        wannaTracePossibleJumps = 0;
    }

    //afterwards, draw player
    mvaddch( pY, pX, g_player );
    if( wasHurt == 1 )
    {
        wasHurt = 0;
        mvaddch( pY, pX, '@' | COLOR_PAIR( 32 ) | A_BOLD );
    }
}

void trace_possible_jump( int y, int x )
{
        int spot = -1;
        spot = map_collision( y, x );
        if( spot != A_WALL )
        {
            if( spot == A_MINE )
                mvaddch( y, x, 'X' | COLOR_PAIR( 8 ) | A_BOLD );
            else if( spot == A_SANDWICH )
                mvaddch( y, x, '=' | COLOR_PAIR( 8 ) | A_BOLD );
            else
                mvaddch( y, x, '@' | COLOR_PAIR( 1 ) );
        }
}

int load_screen_from_text_file()
{
    FILE *fp = NULL;

    fp = fopen( "boards.txt", "r" );
    if( fp == NULL )
    {
        mvprintw( 0, 0, "Error: couldn't access boards.txt for reading." );
        mvprintw( 1, 0, "press any key" );
        getch();
        return 1;
    }

    int z, y, x, in = 0;
    while( in != EOF )
    {
        for( z = 0; z < MAX_STORED_MAPS; z++ )
        {
            for( y = 0; y < SCREEN_Y; y++ )
            {
                for( x = 0; x < SCREEN_X; x++ )
                {
                    in = fgetc( fp );       //read from file
                    while( in == '\n' )     //skip over newlines
                    {
                        in = fgetc( fp );
                    }
                    //should really be using variables that hold these symbols but whatever
                    if( in == '.' )
                    {
                        mapStore[ z ][ y ][ x ] = A_BLANK;
                    }
                    if( in == '&' )
                    {
                        mapStore[ z ][ y ][ x ] = A_WALL;
                    }
                    if( in == '=' )
                    {
                        mapStore[ z ][ y ][ x ] = A_SANDWICH;
                    }
                    if( in == 'X' )
                    {
                        mapStore[ z ][ y ][ x ] = A_MINE;
                    }
                    if( in == '@' )
                    {
                        mapStore[ z ][ y ][ x ] = A_PLAYER_START;
                    }
                }
            }
        }
    }
    fclose( fp );
    return 0;
}

int save_screen_to_text_file()
{
    FILE *fp = NULL;

    fp = fopen( "boards.txt", "w" );
    if( fp == NULL )
    {
        mvprintw( 0, 0, "Error: couldn't access boards.txt for writing." );
        mvprintw( 1, 0, "press any key" );
        getch();
        return 1;
    }
    int z, y, x;
    for( z = 0; z < MAX_STORED_MAPS; z++ )
    {
        for( y = 0; y < SCREEN_Y; y++ )
        {
            for( x = 0; x < SCREEN_X; x++ )
            {
                switch( mapStore[ z ][ y ][ x ] )
                {
                    case A_BLANK:
                    {
                        fputc( g_empty, fp );
                        break;
                    }
                    case A_WALL:
                    {
                        fputc( g_wall, fp );
                        break;
                    }
                    case A_SANDWICH:
                    {
                        fputc( g_sandwich, fp );
                        break;
                    }
                    case A_MINE:
                    {
                        fputc( g_mine, fp );
                        break;
                    }
                    case A_PLAYER_START:
                    {
                        fputc( g_player, fp );
                        break;
                    }
                }
            }
            fputc( '\n', fp );
        }
        fputc( '\n', fp );
    }
    fclose( fp );
    return 0;
}

void logic()
{
    blank_previous_coordinates();       //pPrev_

    pFinishInput = 0;
    pFallDamage = -5;

    int doneMoving = 0;
    while( doneMoving == 0 )
    {
        //get input
        if( pFinishInput == 0 )
        {
            if( pInput != 0 )
            {
                switch( pInput )
                {
                    case MOVE_MEASURE:
                    {
                        wannaTracePossibleJumps = 1;
                        break;
                    }
                    case MOVE_STOP:
                    {
                        break;
                    }
                    case MOVE_LEFT:
                    {
                        pMoveSide = -1;
                        break;
                    }
                    case MOVE_RIGHT:
                    {
                        pMoveSide = 1;
                        break;
                    }
                    case MOVE_JUMP_LEFT:
                    {
                        pMoveRise = 1;
                        pMoveSide = -( 3 + pMoveAugment );
                        break;
                    }
                    case MOVE_JUMP_RIGHT:
                    {
                        pMoveRise = 1;
                        pMoveSide = ( 3 + pMoveAugment );
                        break;
                    }
                    case MOVE_HI_JUMP_LEFT:
                    {
                        pMoveRise = ( 3 + pMoveAugment );
                        pMoveSide = -2;
                        break;
                    }
                    case MOVE_HI_JUMP_RIGHT:
                    {
                        pMoveRise = ( 3 + pMoveAugment );
                        pMoveSide = 2;
                        break;
                    }
                    case MOVE_HI_JUMP:
                    {
                        pMoveRise = ( 4 + pMoveAugment );
                        break;
                    }
                }
            pFinishInput = 1;
            pInput = 0;
            }
        }

        //putting the win condition before movement to correct
        //an error where you could grab all sandwiches but still
        //die on a mine
        int y, x;
        int eatenAllSandwiches = 1;
        for( y = 0; y < SCREEN_Y; y++ )
        {
            for( x = 0; x < SCREEN_X; x++ )
            {
                if( map[ y ][ x ] == A_SANDWICH )
                {
                    eatenAllSandwiches = 0;
                }
            }
        }
        if( eatenAllSandwiches == 1 )
        {
            if( mapIndex < MAX_STORED_MAPS - 1 )
            {
                transition( WON_LEVEL );
                return;
            }
            else if( mapIndex == MAX_STORED_MAPS - 1 )
            {
                transition( WON_GAME );
                return;
            }
        }

        //beginning of actual movement.

        //check for collision with sandwiches and mines.
        if( map_collision( pY, pX ) == A_SANDWICH )
        {
            map[ pY ][ pX ] = A_BLANK;
            pHealth += 10;
            if( pHealth > healthMax )
                pHealth = healthMax;
            pMoveAugment++;
            continue;
        }
        if( map_collision( pY, pX ) == A_MINE )
        {
            transition( TRIPPED_A_MINE );
            return;
        }

        //actually move the player. first up, then left-or-right
        if( pMoveRise > 0 )
        {
            pMoveRise--;
            if( map_collision( pY - 1, pX ) != A_WALL )
            {
                record_previous_coordinates();
                pY--;
            }
            continue;
        }
        if( pMoveSide > 0 )
        {
            pMoveSide--;
            if( map_collision( pY, pX + 1 ) != A_WALL )
            {
                record_previous_coordinates();
                pX++;
            }
            continue;
        }
        if( pMoveSide < 0 )
        {
            pMoveSide++;
            if( map_collision( pY, pX - 1 ) != A_WALL )
            {
                record_previous_coordinates();
                pX--;
            }
            continue;
        }

    //IF the player does not have any outstanting move/jump commnads:
            //check if the player has any footing on the ground.
            //if not, fall like a rock
                              //A ROCK, MOTHERFUCKER
            //and record coordinates for previous coord array
            //keep track of how far you fall; add fall damage afterwards

            if( pMoveRise == 0 && pMoveSide == 0 && (map_collision( pY + 1, pX ) != A_WALL) )
            {
                record_previous_coordinates();

                pY++;
                pFallDamage++;
                //mark point where player begins to take fall damage
                if( pFallDamage > 0  && pPrevHurt > P_PREV_MAX )
                {
                    pPrevHurt = pPrevLen;
                }
                pFinishInput = 1;
                //back to top!
                continue;
            }

    //reaching this point = all finished moving.
    //sub fall damage (if there is any) from health
    if( pFallDamage > 0 )
    {
        pHealth -= pFallDamage;
        wasHurt = 1;
    }
    doneMoving = 1;
    }
    //increment turns
    pTurns++;

    //lose transitions
        // (starvation removed)
    if( pHealth <= 0 )
    {
        transition( FELL_TO_DEATH );
    }
    return;
}

int map_collision( int y, int x )
{
    if( (y < 0) || (y > SCREEN_Y - 1) || (x < 0) || (x > SCREEN_X - 1) )
    {
        return A_WALL;
    }
    else
    {
        return map[ y ][ x ];
    }
}

void record_previous_coordinates()
{
    pPrevY[ pPrevLen ] = pY;
    pPrevX[ pPrevLen ] = pX;
    pPrevLen++;
}



void draw()
{
    curs_set( 0 );
    draw_screen();

    color_set( 7, NULL );
    mvprintw( 24, 0, "Turns: %d", pTurns );

    mvprintw( 24, 11, "Health: %d/40", pHealth );

    mvprintw( 24, 27, "Jump: %d", pMoveAugment );

    color_set( 4, NULL );
    mvprintw( 24, 40, "Reset, Quit, /Edit" );
    mvaddch( 24, 40, 'R' | COLOR_PAIR( 4 ) | A_BOLD );
    mvaddch( 24, 47, 'Q' | COLOR_PAIR( 4 ) | A_BOLD );
    mvaddch( 24, 53, '/' | COLOR_PAIR( 4 ) | A_BOLD );


    return;
}

void input()
{
    int keyRetry = 1;
    while( keyRetry == 1 )
    {
        keyRetry = 0;
        keyIn = getch();
        switch( keyIn )
        {
            case 'Q':
            case 'q':
            {
                quit = 1;
                break;
            }
            case 'R':
            case 'r':
            {
                reset_level();
                break;
            }
            case '/':
            {
                change_app_mode( EDITING );
                break;
            }
            case '0':
            {
                pInput = MOVE_MEASURE;
                break;
            }
            case '1':
            {
                pInput = MOVE_LEFT;
                break;
            }
            case '2':
            {
                pInput = MOVE_STOP;
                break;
            }
            case '3':
            {
                pInput = MOVE_RIGHT;
                break;
            }
            case '4':
            {
                pInput = MOVE_JUMP_LEFT;
                break;
            }
            case '6':
            {
                pInput = MOVE_JUMP_RIGHT;
                break;
            }
            case '7':
            {
                pInput = MOVE_HI_JUMP_LEFT;
                break;
            }
            case '8':
            {
                pInput = MOVE_HI_JUMP;
                break;
            }
            case '9':
            {
                pInput = MOVE_HI_JUMP_RIGHT;
                break;
            }
            default:
            keyRetry = 1;
        }
    }
}

void play()
{
    logic();
    draw();
    if( isItOverYet == 0 )      // to fix a small cosmetic goof, wherein, on winning, the program flips back to the game board before exiting.
    {
        input();
    }
}

void logic_edit()
{
    blank_previous_coordinates(); //gets rid of the motion indicators

    if( wannaSave == 1 )
    {
        mvprintw( 0, 0, "Confirm save with Y (upper case)" );
        int confirm = getch();
        if( confirm == 'Y' )
        {
            int test = save_screen_to_text_file();
            if( test != 0 )
            {
                mvprintw( 3, 0, "saving error" );
                getch();
            }
            else
            {
                int i;
                for( i = 0; i < 80; i++ )
                {
                    mvaddch( 0, i, '_' );
                }
                mvprintw( 0, 0, "Save successful" );
                getch();
            }
            reset_level();
        }
        wannaSave = 0;
    }
    if( wannaLoad == 1 )
    {
        mvprintw( 0, 0, "Confirm load with Y (upper case)" );
        int confirm = getch();
        if( confirm == 'Y' )
        {
            int test = load_screen_from_text_file();
            if( test != 0 )
            {
                mvprintw( 3, 0, "loading error" );
            }
            reset_level();
        }
        wannaLoad = 0;
    }

    if( ePaintMode == 1 )
    {
        wannaPlot = 1;
    }

    if( wannaFill == 1 )
    {
        flood_fill();
        wannaFill = 0;
        //wannaPlot = 0;
        reset_level();
    }

    if( wannaGrab == 1 )
    {
        eThingIndex = mapStore[ mapIndex ][ eY ][ eX ];
        wannaGrab = 0;
    }

    while( eYMove != 0 )
    {
        if( eYMove > 0 )
        {
            eY++;
            eYMove--;
            if( eY > SCREEN_Y - 1 )
                eY = SCREEN_Y - 1;
            check_plot_thing();
        }
        if( eYMove < 0 )
        {
            eY--;
            eYMove++;
            if( eY < 0 )
                eY = 0;
            check_plot_thing();
        }
    }
    while( eXMove != 0 )
    {
        if( eXMove > 0 )
        {
            eX++;
            eXMove--;
            if( eX > SCREEN_X - 1 )
                eX = SCREEN_X - 1;
            check_plot_thing();
        }
        if( eXMove < 0 )
        {
            eX--;
            eXMove++;
            if( eX < 0 )
                eX = 0;
            check_plot_thing();
        }
    }

    check_plot_thing();
    wannaPlot = 0;
    return;
}
void draw_edit()
{
    draw_screen();

    //draw editing statusbar
    attrset( COLOR_PAIR( 7 ) | A_BOLD );
    mvprintw( 24, 0, "L_ [dec ]inc (s)ave (l)oad (q)uit" );

    mvprintw( 24, 1, "%d", mapIndex + 1 );

    mvprintw( 24, 35, " . & X = @  zx:pick tab:paint f:fill ctrl:fst" );
    mvprintw( 24, 35 + eThingIndex * 2, ">" );



    // Confirm with 'Y'. All other keys will cancel.

    if( ePaintMode == 0 )
    {
        curs_set( 1 );
    }
    if( ePaintMode == 1 )
    {
        curs_set( 2 );
    }
    move( eY, eX );

    return;
}
void input_edit()
{
    int keyRetry = 1;
    while( keyRetry == 1 )
    {
        keyRetry = 0;
        keyIn = getch();
        switch( keyIn )
        {
            case 'Q':
            case 'q':
            {
                quit = 1;
                break;
            }
            case 'R':
            case 'r':
            {
                reset_level();
                break;
            }
            case 'S':
            case 's':
            {
                wannaSave = 1;
                break;
            }
            case 'L':
            case 'l':
            {
                wannaLoad = 1;
                break;
            }
            case '/':
            {
                appMode = PLAYING;
                break;
            }
            case KEY_LEFT:
            case '4':
            {
                eXMove = -1;
                break;
            }
            case KEY_RIGHT:
            case '6':
            {
                eXMove = 1;
                break;
            }
            case KEY_UP:
            case '8':
            {
                eYMove = -1;
                break;
            }
            case KEY_DOWN:
            case '2':
            {
                eYMove = 1;
                break;
            }
            case KEYB_CTRL_LEFT:
            {
                eXMove = -eFastMoveDist;
                break;
            }
            case KEYB_CTRL_RIGHT:
            {
                eXMove = eFastMoveDist;
                break;
            }
            case KEYB_CTRL_UP:
            {
                eYMove = -eFastMoveDist;
                break;
            }
            case KEYB_CTRL_DOWN:
            {
                eYMove = eFastMoveDist;
                break;
            }
            case '[':
            {
                if( mapIndex > 0 )
                {
                    mapIndex--;
                    reset_level();
                }
                break;
            }
            case ']':
            {
                if( mapIndex < MAX_STORED_MAPS - 1)
                {
                    mapIndex++;
                    reset_level();
                }
                break;
            }
            case ' ':
            {
                wannaPlot = 1;
                break;
            }
            case 'Z':
            case 'z':
            {
                eThingIndex--;
                if( eThingIndex < 0 )
                    eThingIndex = eThingLen;
                break;
            }
            case 'X':
            case 'x':
            {
                eThingIndex++;
                if( eThingIndex > eThingLen )
                    eThingIndex = 0;
                break;
            }
            case 'C':
            case 'c':
            {
                wannaGrab = 1;
                break;
            }
            case KEYB_TAB:
            {
                ePaintMode = 1 - ePaintMode;
                break;
            }
            case 'F':
            case 'f':
            {
                wannaFill = 1;
                break;
            }
            default:
            {
                keyRetry = 1;
                break;
            }
        }
    }

    return;
}

void edit()
{
    logic_edit();
    draw_edit();
    input_edit();
}

void clean_up()
{
    //close PDCurses
    exit( endwin() );
}

int main()
{
    if( init() != 0)
        return 1;

    if( load_screen_from_text_file() != 0 )
    {
        mvprintw( 0, 0, "Error loading boards.txt. Quitting! RUNNING AWAY" );
        getch();
        return 1;
    }

    title_screen();
    getch();

    reset_level();

    while( quit == 0 )
    {
        if( appMode == PLAYING )
            play();
        else if( appMode == EDITING )
            edit();
    }

    clean_up();
    return 0;
}
